using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Direct3D = Microsoft.DirectX.Direct3D;

//user namespace 
using Object3DLibrary;
using Tierra;

namespace Tierra3D
{
    public class DrawDirectX
    {

        Device device = null;//_OfoCX
        PresentParameters presentParams = new PresentParameters();//foCXp[^
        ObjectLibrary _ObjectLibrary;//IuWFNgCu


        //RXgN^
        public DrawDirectX()
        {

        }

        //OtBbN̏
        public bool InitializeGraphics(Control renderWindows)
        {
            // Get the current desktop display mode, so we can set up a back
            // buffer of the same format
            try
            {
                // Set up the structure used to create the D3DDevice. Since we are now
                // using more complex geometry, we will create a device with a zbuffer.
                presentParams.Windowed = true;
                presentParams.SwapEffect = SwapEffect.Discard;
                presentParams.EnableAutoDepthStencil = true;
                presentParams.AutoDepthStencilFormat = DepthFormat.D16;
                presentParams.MultiSample = MultiSampleType.FourSamples;//D3DMULTISAMPLE_4_SAMPLES

                // Create the D3DDevice
                device = new Device(0, DeviceType.Hardware, renderWindows, CreateFlags.SoftwareVertexProcessing, presentParams);
                device.DeviceReset += new System.EventHandler(this.OnResetDevice);

                this.OnResetDevice(device, null);//foCX̃Zbg

            }
            catch (DirectXException)
            {
                return false;
            }
            return true;
        }

        //foCXj󂳂ꂽƂɌĂ΂镨
        void OnResetDevice(object sender, EventArgs e)
        {
            //Ă΂ꂽƂ̃foCX
            Device dev = (Device)sender;

            //meshtexturefoCXꂽƂ邩蒼Kv@ȉn

            SetRenderState(dev);
            LoadObjects(dev);

        }

        //t@C̃[h
        protected void LoadObjects(Device dev)
        {
            //Cu̍쐬
            _ObjectLibrary = new ObjectLibrary(dev);


            //IuWFNg̍쐬
            for (int i = 0; i < 32; i++)//32@Ԃ񐔂͌ŒH
            {

                //R[hԍuVo
                SolidBrush sbrs = (SolidBrush)TierraOS.getCodeBrush(i);

                Object3D obj;
                Material[] mats = new Material[2];

                //[hƓo^
                _ObjectLibrary.AddLoadObject("cube.x", i.ToString());
                obj = _ObjectLibrary.GetObject(i.ToString());
               

                mats[1].Diffuse = Color.FromArgb(80, 255, 255, 255);//
                mats[0].Diffuse = Color.FromArgb(200, sbrs.Color.R, sbrs.Color.G, sbrs.Color.B);
                obj.SetMaterials(mats);
            }
            //-2̕
            {
                //R[hԍuVo
                SolidBrush sbrs = (SolidBrush)TierraOS.getCodeBrush(-2);

                Object3D obj;
                Material[] mats = new Material[2];

                //[hƓo^
                _ObjectLibrary.AddLoadObject("cube.x", (-2).ToString());
                obj = _ObjectLibrary.GetObject((-2).ToString());

                

                mats[1].Diffuse = Color.FromArgb(80, 255, 255, 255);//
                mats[0].Diffuse = Color.FromArgb(200, sbrs.Color.R, sbrs.Color.G, sbrs.Color.B);
                obj.SetMaterials(mats);              
            }

        }

        //_[Xe[g̐ݒiŒ肷)
        protected void SetRenderState(Device dev)
        {

            // Turn on the zbuffer
            dev.RenderState.ZBufferEnable = true;

            // Turn on ambient lighting 
            dev.RenderState.Ambient = System.Drawing.Color.White;

            // At@ ufBOLɂ
            device.RenderState.AlphaBlendEnable = true;
            // \[X uh Xe[g̐ݒB
            device.RenderState.SourceBlend = Blend.SourceAlpha;
            // fXeBl[V uh Xe[g̐ݒB
            device.RenderState.DestinationBlend = Blend.InvSourceAlpha;

            //}`TvO
            device.RenderState.MultiSampleAntiAlias = true;
            //}XN
            device.RenderState.MultiSampleMask = 0x0000ffff;

            device.RenderState.Lighting = true;

        }


        //CeBÕZbg
        private void SetupLights()
        {
            //P3DIuWFNg͑S_猩̂3Cg͕Kv

            device.Lights[0].Type = LightType.Directional;
            device.Lights[0].Diffuse = Color.FromArgb(255, 255, 255);
            device.Lights[0].Direction = new Vector3(1.0f, 1.0f, 1.0f);


            device.Lights[1].Type = LightType.Directional;
            device.Lights[1].Diffuse = Color.FromArgb(255, 255, 255);
            device.Lights[1].Direction = new Vector3(-1.0f, 1.0f, -1.0f);

            device.Lights[2].Type = LightType.Directional;
            device.Lights[2].Diffuse = Color.FromArgb(255, 255, 255);
            device.Lights[2].Direction = new Vector3(-1.0f, -1.0f, 1.0f);

            device.Lights[3].Type = LightType.Directional;
            device.Lights[3].Diffuse = Color.FromArgb(255, 255, 255);
            device.Lights[3].Direction = new Vector3(0.0f, 1.0f, 0.0f);


            device.Lights[0].Enabled = true;
            device.Lights[1].Enabled = true;
            device.Lights[2].Enabled = true;
            device.Lights[3].Enabled = true;
        
            //ArGg
            device.RenderState.Ambient = System.Drawing.Color.FromArgb(0x202020);


        }

        //J|WVp
        public float cameraYpos = 0;
        public float cameraXpos = 0.0f;
        public float cameraZpos = -5.0f;

        public void Render()
        {
            if (device == null)
                return;


            //Clear the backbuffer to a blue color 
            device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, System.Drawing.Color.Gray, 1.0f, 0);


            //Begin the scene
            device.BeginScene();

            
            
            //J
            device.Transform.View = Matrix.LookAtLH(new Vector3(cameraXpos, cameraYpos, cameraZpos), new Vector3(cameraXpos, cameraYpos, 0.0f), new Vector3(0.0f, 1.0f, 0.0f));


            //vWFNV
            int width = device.PresentationParameters.BackBufferWidth;
            int height = device.PresentationParameters.BackBufferHeight;
            device.Transform.Projection = Matrix.PerspectiveFovLH((float)(Math.PI / 4), width / (float)height, 1.0f, 100.0f);


            //CeBÕZbg
            SetupLights();

            //-----------------------------------------------------//
            //Ƀ_Ô
            Object3D obj1 =  _ObjectLibrary.GetObject(0.ToString());//IuWFNg̓


            Body3D body1 = new Body3D(obj1,device);
            Material[] mats = new Material[2];





            //uVƂc
            for (int i = 0; i < CellRecord.registCellGroup.Count; i++)
            {

                //R[h
                for (int j = 0; j < CellRecord.registCellGroup[i].codeAry.Length; j++)
                {
                    //F̎擾
                    //uV͍v32ނ
                    String str = CellRecord.registCellGroup[i].codeAry[j].ToString();

                    Object3D tmpObj = _ObjectLibrary.GetObject(str);

                    Body3D tempBody = new Body3D(tmpObj,device);
                    SolidBrush sbrs = (SolidBrush)TierraOS.getCodeBrush(CellRecord.registCellGroup[i].codeAry[j]);

                    tempBody.SetAngle(0, (float)Math.Sin((Environment.TickCount + sbrs.Color.ToArgb())/1000.0f), 0);
                    tempBody.SetPosition(i*2,j,0);
                    tempBody.SetScale(0.8f,0.8f,0.8f);

                    tempBody.Draw();
                }
            }
            

 //           float size = (float)(Math.Sin(Environment.TickCount/1000.0)+1)+1;
            body1.SetScale(3,3,3);
            body1.SetAngle(0, Environment.TickCount / 1000.0f, 0);

    //        body1.Draw();
            MyRenderObject();
            //-----------------------------------------------------//


            //End the scene
            device.EndScene();
            device.Present();
        }

        //_Ôw
        protected void MyRenderObject()
        { 
        
        }


    }



    //3DIuWFNgNX
    //mesh͎QƂ̂
    class Body3D
    {
        //ʒuƂ
        Vector3 _pos;
        Vector3 _angle;
        Vector3 _scale;

        Object3D _3Dobj;
        Device _dev;

        //RXgN^
        public Body3D(Object3D obj, Device dev)
        {
            //oƂ쐬
            _dev = dev;
            _3Dobj = obj;

            _pos = new Vector3();
            _angle = new Vector3();
            _scale = new Vector3(); _scale.X = _scale.Y = _scale.Z = 1;

        }

        //`֐
        public void Draw()
        {
            Matrix matAngle = Matrix.RotationX(_angle.X) * Matrix.RotationY(_angle.Y) * Matrix.RotationZ(_angle.Z);

            Matrix mat = Matrix.Transformation(new Vector3(0, 0, 0), Quaternion.Identity, _scale, new Vector3(0, 0, 0), Quaternion.RotationMatrix(matAngle), _pos);

            _3Dobj.Draw(mat, _dev);

        }



        //ȉZb^[
        public void SetPosition(float x, float y, float z)
        {
            _pos.X = x; _pos.Y = y; _pos.Z = z;
        }

        public void SetAngle(float x, float y, float z)
        {
            _angle.X = x; _angle.Y = y; _angle.Z = z;
        }

        public void SetScale(float x, float y, float z)
        {
            _scale.X = x; _scale.Y = y; _scale.Z = z;
        }


    }

}
